@@ -1,6 +1,6 @@
 /* BFD back-end for Renesas H8/300 COFF binaries.
    Copyright 1990, 1991, 1992, 1993, 1994, 1995, 1996, 1997, 1998, 1999,
-   2000, 2001, 2002, 2003
+   2000, 2001, 2002, 2003, 2004
    Free Software Foundation, Inc.
    Written by Steve Chamberlain, <sac@cygnus.com>.
 
@@ -154,13 +154,20 @@ funcvec_hash_newfunc (struct bfd_hash_entry *entry,
   /* Bump the offset at which we store entries in the function
      vector.  We'd like to bump up the size of the vectors section,
      but it's not easily available here.  */
-  if (bfd_get_mach (table->abfd) == bfd_mach_h8300)
-    table->offset += 2;
-  else if (bfd_get_mach (table->abfd) == bfd_mach_h8300h
-	   || bfd_get_mach (table->abfd) == bfd_mach_h8300s)
-    table->offset += 4;
-  else
-    return NULL;
+ switch (bfd_get_mach (table->abfd))
+   {
+   case bfd_mach_h8300:
+   case bfd_mach_h8300hn:
+   case bfd_mach_h8300sn:
+     table->offset += 2;
+     break;
+   case bfd_mach_h8300h:
+   case bfd_mach_h8300s:
+     table->offset += 4;
+     break;
+   default:
+     return NULL;
+   }
 
   /* Everything went OK.  */
   return (struct bfd_hash_entry *) ret;
@@ -659,6 +666,7 @@ h8300_reloc16_extra_cases (bfd *abfd, struct bfd_link_info *link_info,
   bfd_vma value;
   bfd_vma dot;
   int gap, tmp;
+  unsigned char temp_code;
 
   switch (reloc->howto->type)
     {
@@ -768,7 +776,7 @@ h8300_reloc16_extra_cases (bfd *abfd, struct bfd_link_info *link_info,
       src_address += 4;
       break;
 
-    /* A 16bit abolute relocation that was formerlly a 24/32bit
+    /* A 16bit absolute relocation that was formerly a 24/32bit
        absolute relocation.  */
     case R_MOVL2:
       value = bfd_coff_reloc16_get_value (reloc, link_info, input_section);
@@ -940,8 +948,12 @@ h8300_reloc16_extra_cases (bfd *abfd, struct bfd_link_info *link_info,
       if (data[dst_address - 2] != 0x6a)
 	abort ();
 
+      temp_code = data[src_address - 1];
+      if ((temp_code & 0x10) != 0x10)
+	temp_code &= 0xf0;
+
       /* Fix up the opcode.  */
-      switch (data[src_address - 1] & 0xf0)
+      switch (temp_code)
 	{
 	case 0x00:
 	  data[dst_address - 2] = (data[src_address - 1] & 0xf) | 0x20;
@@ -949,6 +961,12 @@ h8300_reloc16_extra_cases (bfd *abfd, struct bfd_link_info *link_info,
 	case 0x80:
 	  data[dst_address - 2] = (data[src_address - 1] & 0xf) | 0x30;
 	  break;
+	case 0x18:
+	  data[dst_address - 2] = 0x7f;
+	  break;
+	case 0x10:
+	  data[dst_address - 2] = 0x7e;
+	  break;
 	default:
 	  abort ();
 	}
@@ -965,8 +983,12 @@ h8300_reloc16_extra_cases (bfd *abfd, struct bfd_link_info *link_info,
       if (data[dst_address - 2] != 0x6a)
 	abort ();
 
+      temp_code = data[src_address - 1];
+      if ((temp_code & 0x30) != 0x30)
+	temp_code &= 0xf0;
+
       /* Fix up the opcode.  */
-      switch (data[src_address - 1] & 0xf0)
+      switch (temp_code)
 	{
 	case 0x20:
 	  data[dst_address - 2] = (data[src_address - 1] & 0xf) | 0x20;
@@ -974,6 +996,12 @@ h8300_reloc16_extra_cases (bfd *abfd, struct bfd_link_info *link_info,
 	case 0xa0:
 	  data[dst_address - 2] = (data[src_address - 1] & 0xf) | 0x30;
 	  break;
+	case 0x38:
+	  data[dst_address - 2] = 0x7f;
+	  break;
+	case 0x30:
+	  data[dst_address - 2] = 0x7e;
+	  break;
 	default:
 	  abort ();
 	}
@@ -1096,6 +1124,7 @@ h8300_reloc16_extra_cases (bfd *abfd, struct bfd_link_info *link_info,
 	if (symbol->flags & BSF_LOCAL)
 	  {
 	    char *new_name = bfd_malloc ((bfd_size_type) strlen (name) + 9);
+
 	    if (new_name == NULL)
 	      abort ();
 
@@ -1124,21 +1153,28 @@ h8300_reloc16_extra_cases (bfd *abfd, struct bfd_link_info *link_info,
 	src_address++;
 
 	/* Now create an entry in the function vector itself.  */
-	if (bfd_get_mach (input_section->owner) == bfd_mach_h8300)
-	  bfd_put_16 (abfd,
-		      bfd_coff_reloc16_get_value (reloc,
-						  link_info,
-						  input_section),
-		      vectors_sec->contents + h->offset);
-	else if (bfd_get_mach (input_section->owner) == bfd_mach_h8300h
-		 || bfd_get_mach (input_section->owner) == bfd_mach_h8300s)
-	  bfd_put_32 (abfd,
-		      bfd_coff_reloc16_get_value (reloc,
-						  link_info,
-						  input_section),
-		      vectors_sec->contents + h->offset);
-	else
-	  abort ();
+	switch (bfd_get_mach (input_section->owner))
+	  {
+	  case bfd_mach_h8300:
+	  case bfd_mach_h8300hn:
+	  case bfd_mach_h8300sn:
+	    bfd_put_16 (abfd,
+			bfd_coff_reloc16_get_value (reloc,
+						    link_info,
+						    input_section),
+			vectors_sec->contents + h->offset);
+	    break;
+	  case bfd_mach_h8300h:
+	  case bfd_mach_h8300s:
+	    bfd_put_32 (abfd,
+			bfd_coff_reloc16_get_value (reloc,
+						    link_info,
+						    input_section),
+			vectors_sec->contents + h->offset);
+	    break;
+	  default:
+	    abort ();
+	  }
 
 	/* Gross.  We've already written the contents of the vector section
 	   before we get here...  So we write it again with the new data.  */
@@ -1304,11 +1340,20 @@ h8300_bfd_link_add_symbols (bfd *abfd, struct bfd_link_info *info)
 
 		  /* Bump the size of the vectors section.  Each vector
 		     takes 2 bytes on the h8300 and 4 bytes on the h8300h.  */
-		  if (bfd_get_mach (abfd) == bfd_mach_h8300)
-		    htab->vectors_sec->_raw_size += 2;
-		  else if (bfd_get_mach (abfd) == bfd_mach_h8300h
-			   || bfd_get_mach (abfd) == bfd_mach_h8300s)
-		    htab->vectors_sec->_raw_size += 4;
+		  switch (bfd_get_mach (abfd))
+		    {
+		    case bfd_mach_h8300:
+		    case bfd_mach_h8300hn:
+		    case bfd_mach_h8300sn:
+		      htab->vectors_sec->_raw_size += 2;
+		      break;
+		    case bfd_mach_h8300h:
+		    case bfd_mach_h8300s:
+		      htab->vectors_sec->_raw_size += 4;
+		      break;
+		    default:
+		      abort ();
+		    }
 		}
 	    }
 	}
