@@ -201,7 +201,7 @@ DESCRIPTION
 	descriptors for other opens), with the supplied @var{fd} used as
 	an initial file descriptor (but subject to closure at any time),
 	call bfd_set_cacheable(bfd, 1) on the returned BFD.  The default
-	is to assume no cacheing; the file descriptor will remain open
+	is to assume no caching; the file descriptor will remain open
 	until <<bfd_close>>, and will not be affected by BFD operations
 	on other files.
 
@@ -1024,7 +1024,7 @@ FUNCTION
 	bfd_create_gnu_debuglink_section
 
 SYNOPSIS
-	struct sec *bfd_create_gnu_debuglink_section
+	struct bfd_section *bfd_create_gnu_debuglink_section
 	  (bfd *abfd, const char *filename);
 
 DESCRIPTION
@@ -1089,7 +1089,7 @@ FUNCTION
 
 SYNOPSIS
 	bfd_boolean bfd_fill_in_gnu_debuglink_section
-	  (bfd *abfd, struct sec *sect, const char *filename);
+	  (bfd *abfd, struct bfd_section *sect, const char *filename);
 
 DESCRIPTION
 
@@ -1105,7 +1105,7 @@ RETURNS
 
 bfd_boolean
 bfd_fill_in_gnu_debuglink_section (bfd *abfd,
-				   struct sec *sect,
+				   struct bfd_section *sect,
 				   const char *filename)
 {
   bfd_size_type debuglink_size;
