@@ -1,5 +1,5 @@
 /* arsup.c - Archive support for MRI compatibility
-   Copyright 1992, 1994, 1995, 1996, 1997, 2000, 2002
+   Copyright 1992, 1994, 1995, 1996, 1997, 2000, 2002, 2003
    Free Software Foundation, Inc.
 
    This file is part of GNU Binutils.
@@ -32,17 +32,14 @@
 #include "filenames.h"
 
 static void map_over_list
-  PARAMS ((bfd *, void (*function) (bfd *, bfd *), struct list *));
-static void ar_directory_doer PARAMS ((bfd *, bfd *));
-static void ar_addlib_doer PARAMS ((bfd *, bfd *));
+  (bfd *, void (*function) (bfd *, bfd *), struct list *);
+static void ar_directory_doer (bfd *, bfd *);
+static void ar_addlib_doer (bfd *, bfd *);
 
 extern int verbose;
 
 static void
-map_over_list (arch, function, list)
-     bfd *arch;
-     void (*function) PARAMS ((bfd *, bfd *));
-     struct list *list;
+map_over_list (bfd *arch, void (*function) (bfd *, bfd *), struct list *list)
 {
   bfd *head;
 
@@ -92,18 +89,13 @@ map_over_list (arch, function, list)
 FILE *outfile;
 
 static void
-ar_directory_doer (abfd, ignore)
-     bfd *abfd;
-     bfd *ignore ATTRIBUTE_UNUSED;
+ar_directory_doer (bfd *abfd, bfd *ignore ATTRIBUTE_UNUSED)
 {
   print_arelt_descr(outfile, abfd, verbose);
 }
 
 void
-ar_directory (ar_name, list, output)
-     char *ar_name;
-     struct list *list;
-     char *output;
+ar_directory (char *ar_name, struct list *list, char *output)
 {
   bfd *arch;
 
@@ -130,7 +122,7 @@ ar_directory (ar_name, list, output)
 }
 
 void
-DEFUN_VOID(prompt)
+prompt (void)
 {
   extern int interactive;
 
@@ -142,7 +134,7 @@ DEFUN_VOID(prompt)
 }
 
 void
-maybequit ()
+maybequit (void)
 {
   if (! interactive)
     xexit (9);
@@ -153,9 +145,7 @@ bfd *obfd;
 char *real_name;
 
 void
-ar_open (name, t)
-  char *name;
-  int t;
+ar_open (char *name, int t)
 {
   char *tname = (char *) xmalloc (strlen (name) + 10);
   const char *bname = lbasename (name);
@@ -219,9 +209,7 @@ ar_open (name, t)
 }
 
 static void
-ar_addlib_doer (abfd, prev)
-     bfd *abfd;
-     bfd *prev;
+ar_addlib_doer (bfd *abfd, bfd *prev)
 {
   /* Add this module to the output bfd.  */
   if (prev != NULL)
@@ -232,9 +220,7 @@ ar_addlib_doer (abfd, prev)
 }
 
 void
-ar_addlib (name, list)
-     char *name;
-     struct list *list;
+ar_addlib (char *name, struct list *list)
 {
   if (obfd == NULL)
     {
@@ -249,13 +235,12 @@ ar_addlib (name, list)
       if (arch != NULL)
 	map_over_list (arch, ar_addlib_doer, list);
 
-      /* Don't close the bfd, since it will make the elements disasppear.  */
+      /* Don't close the bfd, since it will make the elements disappear.  */
     }
 }
 
 void
-ar_addmod (list)
-     struct list *list;
+ar_addmod (struct list *list)
 {
   if (!obfd)
     {
@@ -286,15 +271,14 @@ ar_addmod (list)
 
 
 void
-ar_clear ()
+ar_clear (void)
 {
   if (obfd)
     obfd->archive_head = 0;
 }
 
 void
-ar_delete (list)
-     struct list *list;
+ar_delete (struct list *list)
 {
   if (!obfd)
     {
@@ -336,7 +320,7 @@ ar_delete (list)
 }
 
 void
-ar_save ()
+ar_save (void)
 {
   if (!obfd)
     {
@@ -356,8 +340,7 @@ ar_save ()
 }
 
 void
-ar_replace (list)
-     struct list *list;
+ar_replace (struct list *list)
 {
   if (!obfd)
     {
@@ -423,7 +406,7 @@ ar_replace (list)
 
 /* And I added this one.  */
 void
-ar_list ()
+ar_list (void)
 {
   if (!obfd)
     {
@@ -446,7 +429,7 @@ ar_list ()
 }
 
 void
-ar_end ()
+ar_end (void)
 {
   if (obfd)
     {
@@ -456,8 +439,7 @@ ar_end ()
 }
 
 void
-ar_extract (list)
-     struct list *list;
+ar_extract (struct list *list)
 {
   if (!obfd)
     {
