@@ -52,10 +52,6 @@
 # define DWARF2_ADDR_SIZE(bfd) (bfd_arch_bits_per_address (bfd) / 8);
 #endif
 
-#ifndef TC_DWARF2_EMIT_OFFSET
-# define TC_DWARF2_EMIT_OFFSET  generic_dwarf2_emit_offset
-#endif
-
 #ifdef BFD_ASSEMBLER
 
 #include "subsegs.h"
@@ -157,45 +153,41 @@ static bfd_boolean loc_directive_seen;
 /* Current location as indicated by the most recent .loc directive.  */
 static struct dwarf2_line_info current;
 
-/* Fake label name.  */
-static char const fake_label_name[] = ".L0\001";
-
 /* The size of an address on the target.  */
 static unsigned int sizeof_address;
 
-static void generic_dwarf2_emit_offset PARAMS((symbolS *, unsigned int));
-static struct line_subseg *get_line_subseg PARAMS ((segT, subsegT));
-static unsigned int get_filenum PARAMS ((const char *, unsigned int));
-static struct frag *first_frag_for_seg PARAMS ((segT));
-static struct frag *last_frag_for_seg PARAMS ((segT));
-static void out_byte PARAMS ((int));
-static void out_opcode PARAMS ((int));
-static void out_two PARAMS ((int));
-static void out_four PARAMS ((int));
-static void out_abbrev PARAMS ((int, int));
-static void out_uleb128 PARAMS ((addressT));
-static symbolS *symbol_new_now PARAMS ((void));
-static void set_symbol_value_now PARAMS ((symbolS *));
-static offsetT get_frag_fix PARAMS ((fragS *));
-static void out_set_addr PARAMS ((segT, fragS *, addressT));
-static int size_inc_line_addr PARAMS ((int, addressT));
-static void emit_inc_line_addr PARAMS ((int, addressT, char *, int));
-static void out_inc_line_addr PARAMS ((int, addressT));
-static void relax_inc_line_addr PARAMS ((int, segT, fragS *, addressT,
-					 fragS *, addressT));
-static void process_entries PARAMS ((segT, struct line_entry *));
-static void out_file_list PARAMS ((void));
-static void out_debug_line PARAMS ((segT));
-static void out_debug_aranges PARAMS ((segT, segT));
-static void out_debug_abbrev PARAMS ((segT));
-static void out_debug_info PARAMS ((segT, segT, segT));
+static struct line_subseg *get_line_subseg (segT, subsegT);
+static unsigned int get_filenum (const char *, unsigned int);
+static struct frag *first_frag_for_seg (segT);
+static struct frag *last_frag_for_seg (segT);
+static void out_byte (int);
+static void out_opcode (int);
+static void out_two (int);
+static void out_four (int);
+static void out_abbrev (int, int);
+static void out_uleb128 (addressT);
+static offsetT get_frag_fix (fragS *);
+static void out_set_addr (segT, fragS *, addressT);
+static int size_inc_line_addr (int, addressT);
+static void emit_inc_line_addr (int, addressT, char *, int);
+static void out_inc_line_addr (int, addressT);
+static void relax_inc_line_addr (int, segT, fragS *, addressT,
+				 fragS *, addressT);
+static void process_entries (segT, struct line_entry *);
+static void out_file_list (void);
+static void out_debug_line (segT);
+static void out_debug_aranges (segT, segT);
+static void out_debug_abbrev (segT);
+static void out_debug_info (segT, segT, segT);
 
+#ifndef TC_DWARF2_EMIT_OFFSET
+# define TC_DWARF2_EMIT_OFFSET  generic_dwarf2_emit_offset
+static void generic_dwarf2_emit_offset (symbolS *, unsigned int);
+
 /* Create an offset to .dwarf2_*.  */
 
 static void
-generic_dwarf2_emit_offset (symbol, size)
-     symbolS *symbol;
-     unsigned int size;
+generic_dwarf2_emit_offset (symbolS *symbol, unsigned int size)
 {
   expressionS expr;
 
@@ -204,13 +196,12 @@ generic_dwarf2_emit_offset (symbol, size)
   expr.X_add_number = 0;
   emit_expr (&expr, size);
 }
+#endif
 
 /* Find or create an entry for SEG+SUBSEG in ALL_SEGS.  */
 
 static struct line_subseg *
-get_line_subseg (seg, subseg)
-     segT seg;
-     subsegT subseg;
+get_line_subseg (segT seg, subsegT subseg)
 {
   static segT last_seg;
   static subsegT last_subseg;
@@ -256,12 +247,10 @@ get_line_subseg (seg, subseg)
   return ss;
 }
 
-/* Record an entry for LOC ocurring at OFS within the current fragment.  */
+/* Record an entry for LOC occurring at OFS within the current fragment.  */
 
 void
-dwarf2_gen_line_info (ofs, loc)
-     addressT ofs;
-     struct dwarf2_line_info *loc;
+dwarf2_gen_line_info (addressT ofs, struct dwarf2_line_info *loc)
 {
   struct line_subseg *ss;
   struct line_entry *e;
@@ -295,8 +284,7 @@ dwarf2_gen_line_info (ofs, loc)
 }
 
 void
-dwarf2_where (line)
-     struct dwarf2_line_info *line;
+dwarf2_where (struct dwarf2_line_info *line)
 {
   if (debug_type == DEBUG_DWARF2)
     {
@@ -316,8 +304,7 @@ dwarf2_where (line)
    for, emit a line statement appropriately.  */
 
 void
-dwarf2_emit_insn (size)
-     int size;
+dwarf2_emit_insn (int size)
 {
   struct dwarf2_line_info loc;
 
@@ -348,9 +335,7 @@ dwarf2_emit_insn (size)
    empty one.  */
 
 static unsigned int
-get_filenum (filename, num)
-     const char *filename;
-     unsigned int num;
+get_filenum (const char *filename, unsigned int num)
 {
   static unsigned int last_used, last_used_dir_len;
   const char *file;
@@ -411,6 +396,7 @@ get_filenum (filename, num)
     {
       for (i = 1; i < files_in_use; ++i)
 	if (files[i].dir == dir
+	    && files[i].filename
 	    && strcmp (file, files[i].filename) == 0)
 	  {
 	    last_used = i;
@@ -448,8 +434,7 @@ get_filenum (filename, num)
    If an entry is added to the file table, return a pointer to the filename. */
 
 char *
-dwarf2_directive_file (dummy)
-     int dummy ATTRIBUTE_UNUSED;
+dwarf2_directive_file (int dummy ATTRIBUTE_UNUSED)
 {
   offsetT num;
   char *filename;
@@ -485,8 +470,7 @@ dwarf2_directive_file (dummy)
 }
 
 void
-dwarf2_directive_loc (dummy)
-     int dummy ATTRIBUTE_UNUSED;
+dwarf2_directive_loc (int dummy ATTRIBUTE_UNUSED)
 {
   offsetT filenum, line, column;
 
@@ -538,8 +522,7 @@ dwarf2_directive_loc (dummy)
 }
 
 static struct frag *
-first_frag_for_seg (seg)
-     segT seg;
+first_frag_for_seg (segT seg)
 {
   frchainS *f, *first = NULL;
 
@@ -552,8 +535,7 @@ first_frag_for_seg (seg)
 }
 
 static struct frag *
-last_frag_for_seg (seg)
-     segT seg;
+last_frag_for_seg (segT seg)
 {
   frchainS *f, *last = NULL;
 
@@ -568,8 +550,7 @@ last_frag_for_seg (seg)
 /* Emit a single byte into the current segment.  */
 
 static inline void
-out_byte (byte)
-     int byte;
+out_byte (int byte)
 {
   FRAG_APPEND_1_CHAR (byte);
 }
@@ -577,8 +558,7 @@ out_byte (byte)
 /* Emit a statement program opcode into the current segment.  */
 
 static inline void
-out_opcode (opc)
-     int opc;
+out_opcode (int opc)
 {
   out_byte (opc);
 }
@@ -586,8 +566,7 @@ out_opcode (opc)
 /* Emit a two-byte word into the current segment.  */
 
 static inline void
-out_two (data)
-     int data;
+out_two (int data)
 {
   md_number_to_chars (frag_more (2), data, 2);
 }
@@ -595,8 +574,7 @@ out_two (data)
 /* Emit a four byte word into the current segment.  */
 
 static inline void
-out_four (data)
-     int data;
+out_four (int data)
 {
   md_number_to_chars (frag_more (4), data, 4);
 }
@@ -604,8 +582,7 @@ out_four (data)
 /* Emit an unsigned "little-endian base 128" number.  */
 
 static void
-out_uleb128 (value)
-     addressT value;
+out_uleb128 (addressT value)
 {
   output_leb128 (frag_more (sizeof_leb128 (value, 0)), value, 0);
 }
@@ -613,37 +590,16 @@ out_uleb128 (value)
 /* Emit a tuple for .debug_abbrev.  */
 
 static inline void
-out_abbrev (name, form)
-     int name, form;
+out_abbrev (int name, int form)
 {
   out_uleb128 (name);
   out_uleb128 (form);
 }
 
-/* Create a new fake symbol whose value is the current position.  */
-
-static symbolS *
-symbol_new_now ()
-{
-  return symbol_new (fake_label_name, now_seg, frag_now_fix (), frag_now);
-}
-
-/* Set the value of SYM to the current position in the current segment.  */
-
-static void
-set_symbol_value_now (sym)
-     symbolS *sym;
-{
-  S_SET_SEGMENT (sym, now_seg);
-  S_SET_VALUE (sym, frag_now_fix ());
-  symbol_set_frag (sym, frag_now);
-}
-
 /* Get the size of a fragment.  */
 
 static offsetT
-get_frag_fix (frag)
-     fragS *frag;
+get_frag_fix (fragS *frag)
 {
   frchainS *fr;
 
@@ -655,11 +611,7 @@ get_frag_fix (frag)
      on some subsegment chain.  */
   for (fr = frchain_root; fr; fr = fr->frch_next)
     if (fr->frch_last == frag)
-      {
-	long align_mask = -1 << get_recorded_alignment (fr->frch_seg);
-	return (((char *) obstack_next_free (&fr->frch_obstack)
-		 - frag->fr_literal) + ~align_mask) & align_mask;
-      }
+      return (char *) obstack_next_free (&fr->frch_obstack) - frag->fr_literal;
 
   abort ();
 }
@@ -667,15 +619,12 @@ get_frag_fix (frag)
 /* Set an absolute address (may result in a relocation entry).  */
 
 static void
-out_set_addr (seg, frag, ofs)
-     segT seg;
-     fragS *frag;
-     addressT ofs;
+out_set_addr (segT seg, fragS *frag, addressT ofs)
 {
   expressionS expr;
   symbolS *sym;
 
-  sym = symbol_new (fake_label_name, seg, ofs, frag);
+  sym = symbol_temp_new (seg, ofs, frag);
 
   out_opcode (DW_LNS_extended_op);
   out_uleb128 (sizeof_address + 1);
@@ -688,11 +637,10 @@ out_set_addr (seg, frag, ofs)
 }
 
 #if DWARF2_LINE_MIN_INSN_LENGTH > 1
-static void scale_addr_delta PARAMS ((addressT *));
+static void scale_addr_delta (addressT *);
 
 static void
-scale_addr_delta (addr_delta)
-     addressT *addr_delta;
+scale_addr_delta (addressT *addr_delta)
 {
   static int printed_this = 0;
   if (*addr_delta % DWARF2_LINE_MIN_INSN_LENGTH != 0)
@@ -715,9 +663,7 @@ scale_addr_delta (addr_delta)
    exactly the expected number of bytes.  */
 
 static int
-size_inc_line_addr (line_delta, addr_delta)
-     int line_delta;
-     addressT addr_delta;
+size_inc_line_addr (int line_delta, addressT addr_delta)
 {
   unsigned int tmp, opcode;
   int len = 0;
@@ -776,11 +722,7 @@ size_inc_line_addr (line_delta, addr_delta)
 }
 
 static void
-emit_inc_line_addr (line_delta, addr_delta, p, len)
-     int line_delta;
-     addressT addr_delta;
-     char *p;
-     int len;
+emit_inc_line_addr (int line_delta, addressT addr_delta, char *p, int len)
 {
   unsigned int tmp, opcode;
   int need_copy = 0;
@@ -871,9 +813,7 @@ emit_inc_line_addr (line_delta, addr_delta, p, len)
 /* Handy routine to combine calls to the above two routines.  */
 
 static void
-out_inc_line_addr (line_delta, addr_delta)
-     int line_delta;
-     addressT addr_delta;
+out_inc_line_addr (int line_delta, addressT addr_delta)
 {
   int len = size_inc_line_addr (line_delta, addr_delta);
   emit_inc_line_addr (line_delta, addr_delta, frag_more (len), len);
@@ -883,18 +823,16 @@ out_inc_line_addr (line_delta, addr_delta)
    increments between fragments of the target segment.  */
 
 static void
-relax_inc_line_addr (line_delta, seg, to_frag, to_ofs, from_frag, from_ofs)
-     int line_delta;
-     segT seg;
-     fragS *to_frag, *from_frag;
-     addressT to_ofs, from_ofs;
+relax_inc_line_addr (int line_delta, segT seg,
+		     fragS *to_frag, addressT to_ofs,
+		     fragS *from_frag, addressT from_ofs)
 {
   symbolS *to_sym, *from_sym;
   expressionS expr;
   int max_chars;
 
-  to_sym = symbol_new (fake_label_name, seg, to_ofs, to_frag);
-  from_sym = symbol_new (fake_label_name, seg, from_ofs, from_frag);
+  to_sym = symbol_temp_new (seg, to_ofs, to_frag);
+  from_sym = symbol_temp_new (seg, from_ofs, from_frag);
 
   expr.X_op = O_subtract;
   expr.X_add_symbol = to_sym;
@@ -914,8 +852,7 @@ relax_inc_line_addr (line_delta, seg, to_frag, to_ofs, from_frag, from_ofs)
    the relaxation loop.  We set fr_subtype to the expected length.  */
 
 int
-dwarf2dbg_estimate_size_before_relax (frag)
-     fragS *frag;
+dwarf2dbg_estimate_size_before_relax (fragS *frag)
 {
   offsetT addr_delta;
   int size;
@@ -933,8 +870,7 @@ dwarf2dbg_estimate_size_before_relax (frag)
    of the frag.  This returns the change in frag length.  */
 
 int
-dwarf2dbg_relax_frag (frag)
-     fragS *frag;
+dwarf2dbg_relax_frag (fragS *frag)
 {
   int old_size, new_size;
 
@@ -949,8 +885,7 @@ dwarf2dbg_relax_frag (frag)
    fr_subtype will be the desired length of the frag.  */
 
 void
-dwarf2dbg_convert_frag (frag)
-     fragS *frag;
+dwarf2dbg_convert_frag (fragS *frag)
 {
   offsetT addr_diff;
 
@@ -974,9 +909,7 @@ dwarf2dbg_convert_frag (frag)
    beginning at E, for segment SEG.  */
 
 static void
-process_entries (seg, e)
-     segT seg;
-     struct line_entry *e;
+process_entries (segT seg, struct line_entry *e)
 {
   unsigned filenum = 1;
   unsigned line = 1;
@@ -1068,7 +1001,7 @@ process_entries (seg, e)
 /* Emit the directory and file tables for .debug_line.  */
 
 static void
-out_file_list ()
+out_file_list (void)
 {
   size_t size;
   char *cp;
@@ -1089,6 +1022,8 @@ out_file_list ()
       if (files[i].filename == NULL)
 	{
 	  as_bad (_("unassigned file number %ld"), (long) i);
+	  /* Prevent a crash later, particularly for file 1.  */
+	  files[i].filename = "";
 	  continue;
 	}
 
@@ -1108,8 +1043,7 @@ out_file_list ()
 /* Emit the collected .debug_line data.  */
 
 static void
-out_debug_line (line_seg)
-     segT line_seg;
+out_debug_line (segT line_seg)
 {
   expressionS expr;
   symbolS *line_start;
@@ -1121,9 +1055,9 @@ out_debug_line (line_seg)
 
   subseg_set (line_seg, 0);
 
-  line_start = symbol_new_now ();
-  prologue_end = symbol_make (fake_label_name);
-  line_end = symbol_make (fake_label_name);
+  line_start = symbol_temp_new_now ();
+  prologue_end = symbol_temp_make ();
+  line_end = symbol_temp_make ();
 
   /* Total length of the information for this compilation unit.  */
   expr.X_op = O_subtract;
@@ -1185,21 +1119,19 @@ out_debug_line (line_seg)
 
   out_file_list ();
 
-  set_symbol_value_now (prologue_end);
+  symbol_set_value_now (prologue_end);
 
   /* For each section, emit a statement program.  */
   for (s = all_segs; s; s = s->next)
     process_entries (s->seg, s->head->head);
 
-  set_symbol_value_now (line_end);
+  symbol_set_value_now (line_end);
 }
 
 /* Emit data for .debug_aranges.  */
 
 static void
-out_debug_aranges (aranges_seg, info_seg)
-     segT aranges_seg;
-     segT info_seg;
+out_debug_aranges (segT aranges_seg, segT info_seg)
 {
   unsigned int addr_size = sizeof_address;
   addressT size, skip;
@@ -1247,11 +1179,11 @@ out_debug_aranges (aranges_seg, info_seg)
       symbolS *beg, *end;
 
       frag = first_frag_for_seg (s->seg);
-      beg = symbol_new (fake_label_name, s->seg, 0, frag);
+      beg = symbol_temp_new (s->seg, 0, frag);
       s->text_start = beg;
 
       frag = last_frag_for_seg (s->seg);
-      end = symbol_new (fake_label_name, s->seg, get_frag_fix (frag), frag);
+      end = symbol_temp_new (s->seg, get_frag_fix (frag), frag);
       s->text_end = end;
 
       expr.X_op = O_symbol;
@@ -1275,8 +1207,7 @@ out_debug_aranges (aranges_seg, info_seg)
    sync with out_debug_info below.  */
 
 static void
-out_debug_abbrev (abbrev_seg)
-     segT abbrev_seg;
+out_debug_abbrev (segT abbrev_seg)
 {
   subseg_set (abbrev_seg, 0);
 
@@ -1302,10 +1233,7 @@ out_debug_abbrev (abbrev_seg)
 /* Emit a description of this compilation unit for .debug_info.  */
 
 static void
-out_debug_info (info_seg, abbrev_seg, line_seg)
-     segT info_seg;
-     segT abbrev_seg;
-     segT line_seg;
+out_debug_info (segT info_seg, segT abbrev_seg, segT line_seg)
 {
   char producer[128];
   char *comp_dir;
@@ -1319,8 +1247,8 @@ out_debug_info (info_seg, abbrev_seg, line_seg)
 
   subseg_set (info_seg, 0);
 
-  info_start = symbol_new_now ();
-  info_end = symbol_make (fake_label_name);
+  info_start = symbol_temp_new_now ();
+  info_end = symbol_temp_make ();
 
   /* Compilation Unit length.  */
   expr.X_op = O_subtract;
@@ -1418,11 +1346,11 @@ out_debug_info (info_seg, abbrev_seg, line_seg)
      dwarf2 draft has no standard code for assembler.  */
   out_two (DW_LANG_Mips_Assembler);
 
-  set_symbol_value_now (info_end);
+  symbol_set_value_now (info_end);
 }
 
 void
-dwarf2_finish ()
+dwarf2_finish (void)
 {
   segT line_seg;
   struct line_seg *s;
