@@ -288,6 +288,7 @@
 #define PT_HIPROC	0x7FFFFFFF	/* Processor-specific */
 
 #define PT_GNU_EH_FRAME	(PT_LOOS + 0x474e550)
+#define PT_GNU_STACK	(PT_LOOS + 0x474e551)
 
 /* Program segment permissions, in program header p_flags field.  */
 
@@ -396,6 +397,8 @@
 #define GNU_ABI_TAG_LINUX	0
 #define GNU_ABI_TAG_HURD	1
 #define GNU_ABI_TAG_SOLARIS	2
+#define GNU_ABI_TAG_FREEBSD	3
+#define GNU_ABI_TAG_NETBSD	4
 
 /* Values for NetBSD .note.netbsd.ident notes.  Note name is "NetBSD".  */
 
@@ -530,7 +533,7 @@
    values outside of the new range (see below).	 */
 #define OLD_DT_LOOS	0x60000000
 #define DT_LOOS		0x6000000d
-#define DT_HIOS		0x6fff0000
+#define DT_HIOS		0x6ffff000
 #define OLD_DT_HIOS	0x6fffffff
 
 #define DT_LOPROC	0x70000000
