@@ -49,18 +49,28 @@ print_movxy (op, rn, rm, fprintf_fn, stream)
       switch (op->arg[n])
 	{
 	case A_IND_N:
+	case AX_IND_N:
+	case AXY_IND_N:
+	case AY_IND_N:
+	case AYX_IND_N:
 	  fprintf_fn (stream, "@r%d", rn);
 	  break;
 	case A_INC_N:
+	case AX_INC_N:
+	case AXY_INC_N:
+	case AY_INC_N:
+	case AYX_INC_N:
 	  fprintf_fn (stream, "@r%d+", rn);
 	  break;
-	case A_PMOD_N:
+	case AX_PMOD_N:
+	case AXY_PMOD_N:
 	  fprintf_fn (stream, "@r%d+r8", rn);
 	  break;
-	case A_PMODY_N:
+	case AY_PMOD_N:
+	case AYX_PMOD_N:
 	  fprintf_fn (stream, "@r%d+r9", rn);
 	  break;
-	case DSP_REG_M:
+	case DSP_REG_A_M:
 	  fprintf_fn (stream, "a%c", '0' + rm);
 	  break;
 	case DSP_REG_X:
@@ -69,6 +79,26 @@ print_movxy (op, rn, rm, fprintf_fn, stream)
 	case DSP_REG_Y:
 	  fprintf_fn (stream, "y%c", '0' + rm);
 	  break;
+	case DSP_REG_AX:
+	  fprintf_fn (stream, "%c%c",
+		      (rm & 1) ? 'x' : 'a',
+		      (rm & 2) ? '1' : '0');
+	  break;
+	case DSP_REG_XY:
+	  fprintf_fn (stream, "%c%c",
+		      (rm & 1) ? 'y' : 'x',
+		      (rm & 2) ? '1' : '0');
+	  break;
+	case DSP_REG_AY:
+	  fprintf_fn (stream, "%c%c",
+		      (rm & 2) ? 'y' : 'a',
+		      (rm & 1) ? '1' : '0');
+	  break;
+	case DSP_REG_YX:
+	  fprintf_fn (stream, "%c%c",
+		      (rm & 2) ? 'x' : 'y',
+		      (rm & 1) ? '1' : '0');
+	  break;
 	default:
 	  abort ();
 	}
@@ -102,7 +132,41 @@ print_insn_ddt (insn, info)
   /* Check if either the x or y part is invalid.  */
   if (((insn & 0xc) == 0 && (insn & 0x2a0))
       || ((insn & 3) == 0 && (insn & 0x150)))
-    fprintf_fn (stream, ".word 0x%x", insn);
+    if (info->mach != bfd_mach_sh_dsp
+        && info->mach != bfd_mach_sh3_dsp)
+      {
+	static const sh_opcode_info *first_movx, *first_movy;
+	const sh_opcode_info *op;
+	int is_movy;
+
+	if (! first_movx)
+	  {
+	    for (first_movx = sh_table; first_movx->nibbles[1] != MOVX_NOPY;)
+	      first_movx++;
+	    for (first_movy = first_movx; first_movy->nibbles[1] != MOVY_NOPX;)
+	      first_movy++;
+	  }
+
+	is_movy = ((insn & 3) != 0);
+
+	if (is_movy)
+	  op = first_movy;
+	else
+	  op = first_movx;
+
+	while (op->nibbles[2] != (unsigned) ((insn >> 4) & 3)
+	       || op->nibbles[3] != (unsigned) (insn & 0xf))
+	  op++;
+	
+	print_movxy (op,
+		     (4 * ((insn & (is_movy ? 0x200 : 0x100)) == 0)
+		      + 2 * is_movy
+		      + 1 * ((insn & (is_movy ? 0x100 : 0x200)) != 0)),
+		     (insn >> 6) & 3,
+		     fprintf_fn, stream);
+      }
+    else
+      fprintf_fn (stream, ".word 0x%x", insn);
   else
     {
       static const sh_opcode_info *first_movx, *first_movy;
@@ -191,6 +255,7 @@ print_insn_ppi (field_b, info)
   fprintf_ftype fprintf_fn = info->fprintf_func;
   void *stream = info->stream;
   unsigned int nib1, nib2, nib3;
+  unsigned int altnib1, nib4;
   char *dc = NULL;
   const sh_opcode_info *op;
 
@@ -217,6 +282,16 @@ print_insn_ppi (field_b, info)
 		      sy_tab[(field_b >> 4) & 3],
 		      du_tab[(field_b >> 0) & 3]);
 	}
+      else if ((field_b & 0xf0) == 0x10
+	       && info->mach != bfd_mach_sh_dsp
+	       && info->mach != bfd_mach_sh3_dsp)
+	{
+	  fprintf_fn (stream, "pclr %s \t", du_tab[(field_b >> 0) & 3]);
+	}
+      else if ((field_b & 0xf3) != 0)
+	{
+	  fprintf_fn (stream, ".word 0x%x\t", field_b);
+	}
       fprintf_fn (stream, "pmuls%c%s,%s,%s",
 		  field_b & 0x2000 ? ' ' : '\t',
 		  se_tab[(field_b >> 10) & 3],
@@ -228,6 +303,7 @@ print_insn_ppi (field_b, info)
   nib1 = PPIC;
   nib2 = field_b >> 12 & 0xf;
   nib3 = field_b >> 8 & 0xf;
+  nib4 = field_b >> 4 & 0xf;
   switch (nib3 & 0x3)
     {
     case 0:
@@ -246,14 +322,41 @@ print_insn_ppi (field_b, info)
       nib3 -= 2;
       break;
     }
+  if (nib1 == PPI3)
+    altnib1 = PPI3NC;
+  else
+    altnib1 = nib1;
   for (op = sh_table; op->name; op++)
     {
-      if (op->nibbles[1] == nib1
+      if ((op->nibbles[1] == nib1 || op->nibbles[1] == altnib1)
 	  && op->nibbles[2] == nib2
 	  && op->nibbles[3] == nib3)
 	{
 	  int n;
 
+	  switch (op->nibbles[4])
+	    {
+	    case HEX_0:
+	      break;
+	    case HEX_XX00:
+	      if ((nib4 & 3) != 0)
+		continue;
+	      break;
+	    case HEX_1:
+	      if ((nib4 & 3) != 1)
+		continue;
+	      break;
+	    case HEX_00YY:
+	      if ((nib4 & 0xc) != 0)
+		continue;
+	      break;
+	    case HEX_4:
+	      if ((nib4 & 0xc) != 4)
+		continue;
+	      break;
+	    default:
+	      abort ();
+	    }
 	  fprintf_fn (stream, "%s%s\t", dc, op->name);
 	  for (n = 0; n < 3 && op->arg[n] != A_END; n++)
 	    {
@@ -294,7 +397,7 @@ print_insn_sh (memaddr, info)
 {
   fprintf_ftype fprintf_fn = info->fprintf_func;
   void *stream = info->stream;
-  unsigned char insn[2];
+  unsigned char insn[4];
   unsigned char nibs[4];
   int status;
   bfd_vma relmask = ~(bfd_vma) 0;
@@ -331,8 +434,16 @@ print_insn_sh (memaddr, info)
       target_arch = arch_sh3e;
       break;
     case bfd_mach_sh4:
+    case bfd_mach_sh4_nofpu:
       target_arch = arch_sh4;
       break;
+    case bfd_mach_sh4a:
+    case bfd_mach_sh4a_nofpu:
+      target_arch = arch_sh4a;
+      break;
+    case bfd_mach_sh4al_dsp:
+      target_arch = arch_sh4al_dsp;
+      break;
     case bfd_mach_sh5:
 #ifdef INCLUDE_SHMEDIA
       status = print_insn_sh64 (memaddr, info);
@@ -514,18 +625,21 @@ print_insn_sh (memaddr, info)
 	      fprintf_fn (stream, "r%d", rn);
 	      break;
 	    case A_INC_N:
+	    case AS_INC_N:
 	      fprintf_fn (stream, "@r%d+", rn);
 	      break;
 	    case A_DEC_N:
+	    case AS_DEC_N:
 	      fprintf_fn (stream, "@-r%d", rn);
 	      break;
 	    case A_IND_N:
+	    case AS_IND_N:
 	      fprintf_fn (stream, "@r%d", rn);
 	      break;
 	    case A_DISP_REG_N:
 	      fprintf_fn (stream, "@(%d,r%d)", imm, rn);
 	      break;
-	    case A_PMOD_N:
+	    case AS_PMOD_N:
 	      fprintf_fn (stream, "@r%d+r8", rn);
 	      break;
 	    case A_REG_M:
